# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/jira/auth.ipynb.

# %% auto 0
__all__ = ['JiraAuth', 'JiraAuthPat']

# %% ../../nbs/jira/auth.ipynb 2
import urllib.parse as parse
from dataclasses import dataclass, field
import base64
import httpx

import domolibrary_extensions.client as gd

# %% ../../nbs/jira/auth.ipynb 4
@dataclass
class _JiraAuth_Required:
    instance: str  # 'your-domain.atlassian.net'


@dataclass
class _JiraAuth_Optional:
    base_url: str = None
    is_datacenter: bool = False

# %% ../../nbs/jira/auth.ipynb 5
@dataclass
class JiraAuth(_JiraAuth_Optional, _JiraAuth_Required, gd.Auth):
    pass

# %% ../../nbs/jira/auth.ipynb 6
@dataclass
class _JiraAuthPat_Required:
    email: str
    pat: str = field(repr=False)

    def __post_init__(self):
        self.pat = self.pat.strip()

# %% ../../nbs/jira/auth.ipynb 7
@dataclass
class JiraAuthPat(_JiraAuth_Optional, _JiraAuthPat_Required, _JiraAuth_Required):
    def __post_init__(self):
        if not self.instance.startswith("https://"):
            self.instance = f"https://{self.instance}"

        self.base_url = self.instance
        self.instance = parse.urlparse(self.instance).netloc
        print(self.instance)

    def generate_auth_header(self):
        if self.is_datacenter:
            return {"Authorization": f"Bearer {self.pat}"}

        auth_str = base64.b64encode(f"{self.email}:{self.pat}".encode("ascii")).decode(
            "ascii"
        )
        return {"Authorization": f"Basic {auth_str}"}

    async def who_am_i(self, client: httpx.AsyncClient = None, debug_api: bool = False):
        url = f"{self.base_url}/rest/api/2/myself"

        response = await gd.get_data(
            method="GET",
            url=url,
            auth=self,
            debug_api=debug_api,
            client=client,
        )
        return response



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/google/GAuth.ipynb.

# %% auto 0
__all__ = ['DEFAULT_SCOPE', 'GAuth_Required', 'GoogleAuth_NoEnv_Error', 'generate_creds_from_env', 'generate_creds_from_file',
           'GoogleAuth']

# %% ../../nbs/google/GAuth.ipynb 3
import os
import json
from dataclasses import dataclass, field
from typing import List

import dotenv

from google.auth.transport.requests import Request
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow

# %% ../../nbs/google/GAuth.ipynb 4
from google.oauth2.credentials import Credentials
from googleapiclient.discovery import Resource

# %% ../../nbs/google/GAuth.ipynb 6
def str_to_obj(obj_to_test):
    if isinstance(obj_to_test, str):
        import orjson

        return orjson.loads(obj_to_test)
    return obj_to_test


def _output_creds(creds, output_path=None, env_path=None, token_env_key=None):
    if token_env_key:
        os.environ[token_env_key] = json.dumps(creds.to_json())

    if output_path:
        with open(output_path, "w", encoding="utf-8") as f:
            f.write(creds.to_json())

    if env_path:
        dotenv.set_key(env_path, token_env_key, creds.to_json())

# %% ../../nbs/google/GAuth.ipynb 7
DEFAULT_SCOPE = [
    "https://www.googleapis.com/auth/drive",
    "https://www.googleapis.com/auth/drive.file",
    "https://www.googleapis.com/auth/drive.metadata.readonly",
    "https://www.googleapis.com/auth/drive.readonly",
]

# %% ../../nbs/google/GAuth.ipynb 8
class GAuth_Required(Exception):
    def __init__(self):
        message = f"must pass valid Google Auth or Service with `service_name = docs`"
        super().__init__(message)


class GoogleAuth_NoEnv_Error(Exception):
    def __init__(self, env_token):
        super().__init__(f"{env_token} not found in ENV")


def generate_creds_from_env(
    client_env_key,
    token_env_key=None,
    scope: List[str] = None,
    env_path=None,
):

    scope = scope or DEFAULT_SCOPE

    if env_path:
        print(f"reloading env - {env_path}")
        dotenv.load_dotenv(env_path, override=True)

    token_obj = json.loads(os.environ.get(token_env_key)) if token_env_key else None

    # print(
    #     token_env_key,
    #     "after should be dict",
    #     type(token_obj),
    #     token_obj,
    # )

    creds = (
        Credentials.from_authorized_user_info(token_obj, scope) if token_obj else None
    )

    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())

        else:
            if not os.getenv(client_env_key):
                raise GoogleAuth_NoEnv_Error(client_env_key)

            client_obj = str_to_obj(os.getenv(client_env_key))

            flow = InstalledAppFlow.from_client_config(client_obj, scope)
            creds = flow.run_local_server(port=0)

    _output_creds(creds, env_path=env_path, token_env_key=token_env_key)

    return creds

# %% ../../nbs/google/GAuth.ipynb 10
def generate_creds_from_file(client_path, token_path, scope: List[str] = None):

    scope = scope or DEFAULT_SCOPE

    if os.path.exists(token_path):
        creds = Credentials.from_authorized_user_file(token_path, scope)

    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(client_path, scope)
            creds = flow.run_local_server(port=0)

    _output_creds(creds, output_path=token_path)

    return creds

# %% ../../nbs/google/GAuth.ipynb 11
@dataclass
class GoogleAuth:
    creds: Credentials = None
    service: Resource = None

    scope: List[str] = field(default_factory=lambda: DEFAULT_SCOPE)

    def __post_init__(self):
        self.service = self.service or self.generate_service(
            service_name="drive", service_version="v3"
        )

    @classmethod
    def generate_from_env(
        cls,
        client_env_key,
        token_env_key=None,
        env_path=None,
        scope: List[str] = None,
    ):
        """
        credentials should be stored in the .env file as a json (NOT A STRING)
        token will be stored as a string (NOT A JSON OBJ)

        generate creds here, https://console.cloud.google.com/apis/credentials/oauthclient
        for a new implementation you'll need to
        1. set up a project at console.cloud.google.com,
        2. enable (turn on) the appropriate APIs for that project
        3. configure OAuth access (your python script will use an Oauth token to authenticate, so your project needs the OAuth screen)
        4. download the creds file and store it as env_creds.json
        """

        creds = generate_creds_from_env(
            client_env_key=client_env_key,
            token_env_key=token_env_key,
            scope=scope,
            env_path=env_path,
        )

        return cls(creds=creds)

    @classmethod
    def generate_from_file(
        cls, client_path: str, token_path: str, scope: List[str] = None
    ):
        """generate creds here, https://console.cloud.google.com/apis/credentials/oauthclient
        for a new implementation you'll need to
        1. set up a project at console.cloud.google.com,
        2. enable (turn on) the appropriate APIs for that project
        3. configure OAuth access (your python script will use an Oauth token to authenticate, so your project needs the OAuth screen)
        4. download the client json file and store it as env_creds.json
        """

        creds = generate_creds_from_file(
            client_path=client_path, token_path=token_path, scope=scope
        )

        return cls(creds=creds)

    def generate_service(self, service_name="drive", service_version="v3") -> Resource:
        """Generates a service object for the given service name and version"""

        print(
            f"generating {service_name} - {service_version} service object on {self.__class__.__name__}"
        )

        self.service = build(
            serviceName=service_name, version=service_version, credentials=self.creds
        )
        return self.service



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/execution.ipynb.

# %% auto 0
__all__ = ['gather_with_concurrency']

# %% ../../nbs/utils/execution.ipynb 3
import asyncio

# %% ../../nbs/utils/execution.ipynb 5
async def gather_with_concurrency(
    *coros,  # list of coroutines to await
    n=60,  # number of open coroutines
):
    """limits the number of open coroutines at a time."""

    semaphore = asyncio.Semaphore(n)

    async def sem_coro(coro):
        async with semaphore:
            return await coro

    return await asyncio.gather(*(sem_coro(c) for c in coros))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/google/GSearch.ipynb.

# %% auto 0
__all__ = ['GoogleSearchAuth', 'GoogleSearch']

# %% ../../nbs/google/GSearch.ipynb 2
from typing import List
from dataclasses import dataclass, field
import domolibrary_extensions.google.GAuth as gd
import httpx

from nbdev.showdoc import patch_to

# %% ../../nbs/google/GSearch.ipynb 4
@dataclass
class GoogleSearchAuth(gd.GoogleAuth):
    search_key: str = None
    search_engine_id: str = None
    base_url: str = None

    def __post_init__(self):
        self.base_url = f"https://www.googleapis.com/customsearch/v1?cx={self.search_engine_id}&key={self.search_key}"

    def generate_auth_header(self) -> dict:
        return self.auth_header

# %% ../../nbs/google/GSearch.ipynb 5
@dataclass
class GoogleSearch:
    auth: GoogleSearchAuth = field(repr=False)
    query: str
    is_from_cache: bool
    cnt_results: int
    content: List[dict]
    raw_content: dict = field(repr=False)

    @classmethod
    async def search(
        cls,
        auth: GoogleSearchAuth,
        query: str,
        cache_path: str = None,
        # return_raw: bool = False,
        is_ignore_cache: bool = False,
        debug_api: bool = False,
        client: httpx.AsyncClient = None,
    ):
        res = await gd.get_data(
            url=auth.base_url,
            method="GET",
            cache_path=cache_path,
            debug_api=debug_api,
            params={"q": query},
            is_ignore_cache=is_ignore_cache,
            client=client,
        )

        return cls(
            auth=auth,
            query=query,
            is_from_cache=res.is_from_cache,
            cnt_results=len(res.response["items"]),
            raw_content=res.response,
            content=res.response["items"],
        )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/asana/02_task.ipynb.

# %% ../../nbs/asana/02_task.ipynb 2
from __future__ import annotations

from dataclasses import dataclass, field
from typing import List, Optional

import datetime as dt

from mdutils.mdutils import MdUtils

from nbdev.showdoc import patch_to

import domolibrary_extensions.utils.utils as deut
import domolibrary_extensions.utils.convert as decv
import domolibrary_extensions.client as gd
import domolibrary_extensions.asana.auth as aa
import domolibrary_extensions.asana.user as au
import domolibrary_extensions.asana.project as ap

# %% auto 0
__all__ = ['AsanaSection', 'AsanaMembership', 'AsanaStory', 'AsanaTask']

# %% ../../nbs/asana/02_task.ipynb 4
@dataclass
class AsanaSection:
    id: str
    name: str
    resource_type: str
    auth: aa.AsanaAuth = field(repr=False)

    @classmethod
    def _from_json(cls, data: dict, auth=auth) -> "AsanaSection":
        return cls(
            id=data.get("gid"),
            name=data.get("name"),
            resource_type=data.get("resource_type"),
            auth=auth,
        )


@dataclass
class AsanaMembership:
    """relates a section to a project"""

    project: ap.AsanaProject
    section: AsanaSection
    auth: aa.AsanaAuth = field(repr=False)

    @classmethod
    def _from_json(cls, data: dict, auth: aa.AsanaAuth) -> "AsanaMembership":
        project = (
            ap.AsanaProject._from_json(data["project"], auth=auth)
            if data.get("project")
            else None
        )
        section = (
            AsanaSection._from_json(data["section"], auth=auth)
            if data.get("section")
            else None
        )
        return cls(project=project, section=section, auth=auth)

    def to_text(self):
        return f"{self.project.name} -> {self.section.name}"


@dataclass
class AsanaStory:
    id: str
    created_at: dt.DateTime
    created_by: au.AsanaUser
    text: str
    type: str
    resource_subtype: str

    auth: aa.AsanaAuth = field(repr=False)

    @classmethod
    def _from_json(cls, data: dict, auth: aa.AsanaAuth) -> AsanaStory:
        created_by = (
            au.AsanaUser._from_json(data["created_by"], auth=auth)
            if data.get("created_by")
            else None
        )

        return cls(
            id=data.get("gid"),
            created_at=deut.convert_str_to_date(data.get("created_at")),
            created_by=created_by,
            text=data.get("text"),
            type=data.get("type"),
            resource_subtype=data.get("resource_subtype"),
            auth=auth,
        )

    def to_text(self):
        return f"{self.created_at.date()} - {self.created_by.name} - {self.text}"

    def to_json(self):

        res = {
            **self.__dict__,
            "created_at": decv.convert_datetime_to_str(self.created_at),
            "created_by": self.created_by.name,
        }
        del res["auth"]
        return res

# %% ../../nbs/asana/02_task.ipynb 6
@dataclass
class AsanaTask:
    id: str
    name: str
    workspace_id: str

    auth: aa.AsanaAuth = field(repr=False)
    assignee: au.AsanaUser = None

    assignee_status: str = None

    is_completed: bool = None

    created_at: dt.datetime = None
    completed_on: dt.datetime = None
    due_on: dt.datetime = None
    modified_at: dt.datetime = None

    memberships: List[dict] = None

    notes: str = None

    parent: Optional[dict] = None
    permalink_url: str = None
    projects: List[ap.AsanaProject] = None
    stories: List[AsanaStory] = None

    tags: List[dict] = None

    @classmethod
    def _from_json(cls, obj: dict, auth: aa.AsanaAuth) -> AsanaTask:
        assignee = (
            au.AsanaUser._from_json(obj.get("assignee"), auth=auth)
            if obj.get("assignee")
            else None
        )

        projects = [
            ap.AsanaProject._from_json(project, auth=auth)
            for project in obj.get("projects", [])
        ]

        memberships = [
            AsanaMembership._from_json(member_obj, auth=auth)
            for member_obj in obj.get("memberships", [])
        ]
        return cls(
            id=obj.get("gid"),
            name=obj.get("name"),
            auth=auth,
            workspace_id=obj.get("workspace", {}).get("gid"),
            assignee=assignee,
            is_completed=obj.get("completed"),
            assignee_status=obj.get("assignee_status"),
            completed_on=deut.convert_str_to_date(obj.get("completed_at")),
            created_at=deut.convert_str_to_date(obj.get("created_at")),
            due_on=deut.convert_str_to_date(obj.get("due_on")),
            modified_at=deut.convert_str_to_date(obj.get("modified_at")),
            memberships=memberships,
            notes=obj.get("notes"),
            parent=obj.get("parent"),
            permalink_url=obj.get("permalink_url"),
            tags=obj.get("tags", []),
            projects=projects,
        )

# %% ../../nbs/asana/02_task.ipynb 8
@patch_to(AsanaTask, cls_method=True)
async def get_by_id(
    cls, auth, task_id, debug_api: bool = False, return_raw: bool = False
):
    url = f"{auth.base_url}/tasks/{task_id}"

    res = await gd.get_data(
        auth=auth,
        method="GET",
        url=url,
        debug_api=debug_api,
    )

    if return_raw:
        return res

    return cls._from_json(res.response["data"], auth=auth)

# %% ../../nbs/asana/02_task.ipynb 11
@patch_to(AsanaTask)
async def get_stories(
    self,
    debug_api: bool = False,
    return_raw: bool = False,
    is_only_comments: bool = False,  # stories include any changes to the task, comments are a subtype of stories
):
    auth = self.auth

    url = f"{auth.base_url}/tasks/{self.id}/stories"

    res = await gd.get_data(
        auth=self.auth,
        method="GET",
        url=url,
        debug_api=debug_api,
        # params = {"opt_fields" : "email"}
    )

    if return_raw:
        return res

    self.stories = [
        AsanaStory._from_json(story_obj, auth=auth)
        for story_obj in res.response["data"]
    ]

    if is_only_comments:
        self.stories = [
            story for story in self.stories if "comment" in story.resource_subtype
        ]

    return self.stories

# %% ../../nbs/asana/02_task.ipynb 14
def handle_render_user(mdFile, key, value):
    mdFile.new_line(f"{key} - {value.name}")


def handle_render_datetime(mdFile, key, value):
    mdFile.new_line(f"{key} - {value.date()}")


render_factory_values = {
    "AsanaUser": handle_render_user,
    "datetime": handle_render_datetime,
}


def handle_render_membership(mdFile, key, value):
    mdFile.new_header(level=1, title=key)
    [mdFile.new_line(asana_class.to_text()) for asana_class in value]


def handle_render_stories(mdFile, key, value):
    mdFile.new_header(level=1, title=key)
    [
        mdFile.new_line(asana_class.to_text())
        for asana_class in value
        if "comment" in asana_class.resource_subtype
    ]


def handle_render_default(mdFile, key, value):
    mdFile.new_line(f"{key} - {value}")


render_factory_keys = {
    "memberships": handle_render_membership,
    "stories": handle_render_stories,
    "default": handle_render_default,
}


def render_field(key: str, value: str, mdFile):
    if key in ["name", "projects"] or not value:
        return

    render_fn = (
        render_factory_values.get(value.__class__.__name__)
        or render_factory_keys.get(key)
        or render_factory_keys.get("default")
    )

    render_fn(mdFile=mdFile, key=key, value=value)


@patch_to(AsanaTask)
def to_md(self, output_folder="markdown", output_file=None):
    deut.upsert_folder(output_folder)

    mdFile = MdUtils(
        file_name=f"{output_folder}/{output_file or self.id}",
        title=output_file or self.name,
    )

    [
        render_field(key, getattr(self, key), mdFile)
        for key in self.__dict__.keys()
        if key not in ["auth", "workspace_id"]
    ]
    mdFile.create_md_file()

    return f"done exporting {mdFile.file_name}"
    # mdFile.new_table(columns=3, rows=6, text=list_of_strings, text_align='center')



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/utils.ipynb.

# %% auto 0
__all__ = ['rename_filepath_to_match_datatype', 'detect_encoding', 'read_html_file', 'remove_query_params_from_url', 'update_env',
           'upsert_folder', 'get_all_files_and_folders', 'ImageBlockConverter', 'md', 'convert_html_to_markdown',
           'download_zip', 'download_pptx', 'convert_enum_member_to_name', 'convert_str_to_snake_case',
           'convert_str_remove_accents', 'convert_str_keep_alphanumeric', 'convert_str_file_name',
           'convert_str_to_date']

# %% ../../nbs/utils/utils.ipynb 3
from PIL.Image import Image

# %% ../../nbs/utils/utils.ipynb 4
from enum import Enum
from typing import List, Tuple, Union

import os
import re
import pathlib
import unicodedata
import json
import chardet
from urllib.parse import urljoin, urlparse

from bs4 import BeautifulSoup
from markdownify import MarkdownConverter
import PIL

import datetime as dt

import asyncio

import pptx2md

import zipfile
import io


import datetime as dt
from dateutil.parser import parse as dtu_parse


from nbdev.showdoc import patch_to

# %% ../../nbs/utils/utils.ipynb 7
def rename_filepath_to_match_datatype(data, file_path):
    is_path_ext = os.path.splitext(file_path)[-1].lower()

    old_suffix = pathlib.Path(file_path).suffix if is_path_ext else None

    new_suffix = ""

    if isinstance(data, str) or isinstance(data, bytes) or isinstance(data, bytearray):
        new_suffix = ".txt"
    if isinstance(data, dict):
        new_suffix = ".json"

    file_path = file_path + new_suffix

    if old_suffix:
        file_path = file_path.replace(old_suffix, "")

    return file_path

# %% ../../nbs/utils/utils.ipynb 9
def detect_encoding(file_path, debug_prn: bool = False):
    detector = chardet.universaldetector.UniversalDetector()
    with open(file_path, "rb") as f:
        for line in f:
            detector.feed(line)
            if detector.done:
                break
    detector.close()

    encoding = detector.result

    return encoding

# %% ../../nbs/utils/utils.ipynb 11
def read_html_file(
    file_path, is_convert_to_soup: bool = True
) -> Union[str, BeautifulSoup]:
    if not os.path.exists(file_path):
        raise FileNotFoundError(file_path)

    page_encoding = detect_encoding(file_path)

    with open(file_path, encoding=page_encoding["encoding"]) as fp:
        if is_convert_to_soup:
            return BeautifulSoup(fp, "lxml")

        return fp.read()

# %% ../../nbs/utils/utils.ipynb 13
def remove_query_params_from_url(url):
    u = urlparse(url)
    return urljoin(url, urlparse(url).path)

# %% ../../nbs/utils/utils.ipynb 15
def update_env(env_path: str, key: str, value: str, debug_prn: bool = False) -> dict:
    """
    updates a .env file with a key value pair
    then reloads the env_file
    """

    if not os.path.exists(env_path):
        with open(env_path, "w", encoding="utf-8") as f:
            f.write("")

    quote_mode = "always"

    if isinstance(value, dict):
        quote_mode = "never"
        value = json.dumps(value)

    if debug_prn:
        from pprint import pprint

        pprint(
            {
                "env_path": env_path,
                "key": key,
                "value": value,
                "type": type(value),
                "quote_mode": quote_mode,
            }
        )

    set_key(env_path, key, value, quote_mode=quote_mode)

    set_key(env_path, "env_last_modified", f"updated - {dt.date.today()}")

    load_dotenv(env_path, override=True)

    return {key: os.getenv(key)}

# %% ../../nbs/utils/utils.ipynb 16
def upsert_folder(folder_path: str, debug_prn: bool = False):
    folder_path = os.path.dirname(folder_path)

    if debug_prn:
        print(
            {
                "upsert_folder": os.path.abspath(folder_path),
                "is_exist": os.path.exists(folder_path),
            }
        )

    if not os.path.exists(folder_path):
        os.makedirs(folder_path)

# %% ../../nbs/utils/utils.ipynb 17
def get_all_files_and_folders(
    directory, file_type=None  # to only retrieve a specific file type
) -> Union[Tuple, List]:
    """walk a directory and retrieve a list of files and a list of directory
    returns Tuple of file_ls , dir_ls OR file_ls if file_type supplied
    """
    if not os.path.exists(directory):
        raise FileNotFoundError(directory)

    file_ls = []
    dir_ls = []
    for root, dirs, files in os.walk(directory):
        for name in files:
            if file_type:
                if not name.lower().endswith(file_type.lower()):
                    continue
            file_ls.append(os.path.join(root, name))

        if file_type:
            continue

        for name in dirs:
            dir_ls.append(os.path.join(root, name))

    if file_type:
        return file_ls

    return file_ls, dir_ls

# %% ../../nbs/utils/utils.ipynb 20
class ImageBlockConverter(MarkdownConverter):
    """
    Create a custom MarkdownConverter that adds two newlines after an image
    """

    def convert_img(self, el, text, convert_as_inline, is_resize: bool = True):
        """
        custom image downloader for ImabeBlockConverter
        will handle resize
        """

        if is_resize:
            style_obj = {
                (obj.split(":")[0].strip()): obj.split(":")[1].strip()
                for obj in el.get("style").split(";")
                if ":" in obj
            }

            file_path = os.path.join(
                os.path.dirname(self.options["file_path"]), el["src"]
            )

            image = PIL.Image.open(file_path)

            width = style_obj["width"].replace("px", "")
            width = int(float(width))

            height = style_obj["height"].replace("px", "")
            height = int(float(height))

            new_image = image.resize((width, height))
            new_image.save(file_path)

        return super().convert_img(el, text, convert_as_inline)


def md(html, **options):
    """Create shorthand method for handling conversion"""
    return ImageBlockConverter(**options).convert(html)

# %% ../../nbs/utils/utils.ipynb 21
def convert_html_to_markdown(file_path):
    """converts html file to markdown in place"""

    with open(file_path, encoding="utf-8") as f:
        html = f.read()

    markdown_content = md(
        str(html),
        keep_inline_images_in=["td", "span"],
        file_path=file_path,
        is_resize=True,
    )

    md_path = file_path.replace(".html", ".md")

    with open(md_path, "w+", encoding="utf-8") as f:
        f.write(markdown_content)

    return

# %% ../../nbs/utils/utils.ipynb 22
def download_zip(zip_bytes_content, output_folder, is_convert_to_markdown: bool = True):
    """save bytes content to a zip file then convert html to markdown"""

    zip = zipfile.ZipFile(io.BytesIO(zip_bytes_content), "r")
    zip.extractall(output_folder)

    file_ls = os.listdir(output_folder)

    # rename the html file to index.html
    for file_name in file_ls:
        if file_name.endswith(".html"):
            output_index = os.path.join(output_folder, "index.html")
            os.replace(os.path.join(output_folder, file_name), output_index)

            if is_convert_to_markdown:
                convert_html_to_markdown(os.path.join(output_folder, "index.html"))

    return f"successfully downloaded zip to {output_folder}"

# %% ../../nbs/utils/utils.ipynb 25
def download_pptx(
    pptx_bytes_content, output_folder, is_convert_to_markdown: bool = True
):
    """save bytes content to a pptx file then converts to markdown"""

    upsert_folder(output_folder)

    output_ppt_index = os.path.join(output_folder, "index.pptx")

    with open(output_ppt_index, "wb+") as binary_file:
        # Write bytes to file
        binary_file.write(pptx_bytes_content)

    if is_convert_to_markdown:
        pptx2md.convert(
            output_ppt_index,
            output=os.path.join(output_folder, "index.md"),
            image_dir=os.path.join(output_folder, "images"),
        )

    return f"successfully downloaded content to {output_folder}"

# %% ../../nbs/utils/utils.ipynb 29
def convert_enum_member_to_name(enum_member):
    return enum_member.name if isinstance(enum_member, Enum) else enum_member

# %% ../../nbs/utils/utils.ipynb 30
def convert_str_to_snake_case(text_str):
    """converts 'snake_case_str' to 'snakeCaseStr'"""

    return text_str.replace(" ", "_").lower()

# %% ../../nbs/utils/utils.ipynb 31
def convert_str_remove_accents(text_str: str) -> str:
    return "".join(
        c
        for c in unicodedata.normalize("NFD", text_str)
        if unicodedata.category(c) != "Mn"
    )

# %% ../../nbs/utils/utils.ipynb 33
def convert_str_keep_alphanumeric(text_str) -> str:
    pattern = "[^0-9a-zA-Z_\s]+"

    return re.sub(pattern, "", text_str)

# %% ../../nbs/utils/utils.ipynb 34
def convert_str_file_name(text_str: str) -> str:
    """convert strings to clean file name or url"""

    return convert_str_keep_alphanumeric(
        convert_str_to_snake_case(convert_str_remove_accents(text_str))
    )

# %% ../../nbs/utils/utils.ipynb 36
def convert_str_to_date(datefield: str) -> dt.datetime:
    """converts string date to datetime object"""
    return dtu_parse(datefield) if datefield else None
